@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  */
 #ifndef __CALF_BIQUAD_H
 #define __CALF_BIQUAD_H
@@ -41,12 +41,11 @@ namespace dsp {
  * 
  * don't use this for integers because it won't work
  */
-template<class Coeff = float>
 class biquad_coeffs
 {
 public:
     // filter coefficients
-    Coeff a0, a1, a2, b1, b2;
+    double a0, a1, a2, b1, b2;
     typedef std::complex<double> cfloat;
 
     biquad_coeffs()
@@ -70,16 +69,16 @@ public:
      */
     inline void set_lp_rbj(float fc, float q, float sr, float gain = 1.0)
     {
-        float omega=(float)(2*M_PI*fc/sr);
-        float sn=sin(omega);
-        float cs=cos(omega);
-        float alpha=(float)(sn/(2*q));
-        float inv=(float)(1.0/(1.0+alpha));
+        double omega=(2.0*M_PI*fc/sr);
+        double sn=sin(omega);
+        double cs=cos(omega);
+        double alpha=(sn/(2*q));
+        double inv=(1.0/(1.0+alpha));
 
-        a2 = a0 =  (float)(gain*inv*(1 - cs)*0.5f);
+        a2 = a0 =  (gain*inv*(1.0 - cs)*0.5);
         a1 =  a0 + a0;
-        b1 =  (float)(-2*cs*inv);
-        b2 =  (float)((1 - alpha)*inv);
+        b1 =  (-2.0*cs*inv);
+        b2 =  ((1.0 - alpha)*inv);
     }
 
     // different lowpass filter, based on Zoelzer's equations, modified by
@@ -89,16 +88,16 @@ public:
     // from how it looks, it perhaps uses bilinear transform - but who knows :)
     inline void set_lp_zoelzer(float fc, float q, float odsr, float gain=1.0)
     {
-        Coeff omega=(Coeff)(M_PI*fc*odsr);
-        Coeff omega2=omega*omega;
-        Coeff K=omega*(1+omega2*omega2*Coeff(1.0/1.45));
-        Coeff KK=K*K;
-        Coeff QK=q*(KK+1.f);
-        Coeff iQK=1.0f/(QK+K);
-        Coeff inv=q*iQK;
-        b2 =  (Coeff)(iQK*(QK-K));
-        b1 =  (Coeff)(2.f*(KK-1.f)*inv);
-        a2 = a0 =  (Coeff)(inv*gain*KK);
+        double omega=(M_PI*fc*odsr);
+        double omega2=omega*omega;
+        double K=omega*(1+omega2*omega2*(1.0/1.45));
+        double KK=K*K;
+        double QK=q*(KK+1.f);
+        double iQK=1.0f/(QK+K);
+        double inv=q*iQK;
+        b2 =  (iQK*(QK-K));
+        b1 =  (2.*(KK-1.f)*inv);
+        a2 = a0 =  (inv*gain*KK);
         a1 =  a0 + a0;
     }
 
@@ -110,35 +109,35 @@ public:
      */
     inline void set_hp_rbj(float fc, float q, float esr, float gain=1.0)
     {
-        Coeff omega=(float)(2*M_PI*fc/esr);
-        Coeff sn=sin(omega);
-        Coeff cs=cos(omega);
-        Coeff alpha=(float)(sn/(2*q));
+        double omega=(double)(2*M_PI*fc/esr);
+        double sn=sin(omega);
+        double cs=cos(omega);
+        double alpha=(double)(sn/(2*q));
 
-        float inv=(float)(1.0/(1.0+alpha));
+        double inv=(double)(1.0/(1.0+alpha));
 
-        a0 =  (Coeff)(gain*inv*(1 + cs)/2);
+        a0 =  (gain*inv*(1 + cs)/2);
         a1 =  -2.f * a0;
         a2 =  a0;
-        b1 =  (Coeff)(-2*cs*inv);
-        b2 =  (Coeff)((1 - alpha)*inv);
+        b1 =  (-2*cs*inv);
+        b2 =  ((1 - alpha)*inv);
     }
 
     // this replaces sin/cos with polynomial approximation
     inline void set_hp_rbj_optimized(float fc, float q, float esr, float gain=1.0)
     {
-        Coeff omega=(float)(2*M_PI*fc/esr);
-        Coeff sn=omega+omega*omega*omega*(1.0/6.0)+omega*omega*omega*omega*omega*(1.0/120);
-        Coeff cs=1-omega*omega*(1.0/2.0)+omega*omega*omega*omega*(1.0/24);
-        Coeff alpha=(float)(sn/(2*q));
+        double omega=(double)(2*M_PI*fc/esr);
+        double sn=omega+omega*omega*omega*(1.0/6.0)+omega*omega*omega*omega*omega*(1.0/120);
+        double cs=1-omega*omega*(1.0/2.0)+omega*omega*omega*omega*(1.0/24);
+        double alpha=(double)(sn/(2*q));
 
-        float inv=(float)(1.0/(1.0+alpha));
+        double inv=(double)(1.0/(1.0+alpha));
 
-        a0 =  (Coeff)(gain*inv*(1 + cs)*(1.0/2.0));
+        a0 =  (gain*inv*(1 + cs)*(1.0/2.0));
         a1 =  -2.f * a0;
         a2 =  a0;
-        b1 =  (Coeff)(-2*cs*inv);
-        b2 =  (Coeff)((1 - alpha)*inv);
+        b1 =  (-2.*cs*inv);
+        b2 =  ((1. - alpha)*inv);
     }
     
     /** Bandpass filter based on Robert Bristow-Johnson's equations (normalized to 1.0 at center frequency)
@@ -149,69 +148,69 @@ public:
      */
     inline void set_bp_rbj(double fc, double q, double esr, double gain=1.0)
     {
-        float omega=(float)(2*M_PI*fc/esr);
-        float sn=sin(omega);
-        float cs=cos(omega);
-        float alpha=(float)(sn/(2*q));
+        double omega=(double)(2*M_PI*fc/esr);
+        double sn=sin(omega);
+        double cs=cos(omega);
+        double alpha=(double)(sn/(2*q));
 
-        float inv=(float)(1.0/(1.0+alpha));
+        double inv=(double)(1.0/(1.0+alpha));
 
-        a0 =  (float)(gain*inv*alpha);
+        a0 =  (double)(gain*inv*alpha);
         a1 =  0.f;
-        a2 =  (float)(-gain*inv*alpha);
-        b1 =  (float)(-2*cs*inv);
-        b2 =  (float)((1 - alpha)*inv);
+        a2 =  (double)(-gain*inv*alpha);
+        b1 =  (double)(-2*cs*inv);
+        b2 =  (double)((1 - alpha)*inv);
     }
     
     // rbj's bandreject
     inline void set_br_rbj(double fc, double q, double esr, double gain=1.0)
     {
-        float omega=(float)(2*M_PI*fc/esr);
-        float sn=sin(omega);
-        float cs=cos(omega);
-        float alpha=(float)(sn/(2*q));
+        double omega=(double)(2*M_PI*fc/esr);
+        double sn=sin(omega);
+        double cs=cos(omega);
+        double alpha=(double)(sn/(2*q));
 
-        float inv=(float)(1.0/(1.0+alpha));
+        double inv=(double)(1.0/(1.0+alpha));
 
-        a0 =  (Coeff)(gain*inv);
-        a1 =  (Coeff)(-gain*inv*2*cs);
-        a2 =  (Coeff)(gain*inv);
-        b1 =  (Coeff)(-2*cs*inv);
-        b2 =  (Coeff)((1 - alpha)*inv);
+        a0 =  (gain*inv);
+        a1 =  (-gain*inv*2.*cs);
+        a2 =  (gain*inv);
+        b1 =  (-2.*cs*inv);
+        b2 =  ((1. - alpha)*inv);
     }
     // this is mine (and, I guess, it sucks/doesn't work)
     void set_allpass(float freq, float pole_r, float sr)
     {
-        float a=prewarp(freq, sr);
-        float q=pole_r;
+        double a=prewarp(freq, sr);
+        double q=pole_r;
         set_bilinear(a*a+q*q, -2.0f*a, 1, a*a+q*q, 2.0f*a, 1);
     }
     /// prewarping for bilinear transform, maps given digital frequency to analog counterpart for analog filter design
-    static inline float prewarp(float freq, float sr)
+    static inline double prewarp(float freq, float sr)
     {
         if (freq>sr*0.49) freq=(float)(sr*0.49);
-        return (float)(tan(M_PI*freq/sr));
+        return (double)(tan(M_PI*freq/sr));
     }
     /// convert analog angular frequency value to digital
-    static inline float unwarp(float omega, float sr)
+    static inline double unwarp(float omega, float sr)
     {
-        float T = 1.0 / sr;
+        double T = 1.0 / sr;
         return (2 / T) * atan(omega * T / 2);
     }
     /// convert analog filter time constant to digital counterpart
-    static inline float unwarpf(float t, float sr)
+    static inline double unwarpf(float t, float sr)
     {
         // this is most likely broken and works by pure accident!
-        float omega = 1.0 / t;
+        double omega = 1.0 / t;
         omega = unwarp(omega, sr);
         // I really don't know why does it have to be M_PI and not 2 * M_PI!
-        float f = M_PI / omega;
+        double f = M_PI / omega;
         return f / sr;
     }
     /// set digital filter parameters based on given analog filter parameters
-    void set_bilinear(float aa0, float aa1, float aa2, float ab0, float ab1, float ab2)
+    void set_bilinear(double aa0, double aa1, double aa2, double ab0, double ab1, double ab2)
     {
-        float q=(float)(1.0/(ab0+ab1+ab2));
+        double q=(double)(1.0/(ab0+ab1+ab2));
         a0 = (aa0+aa1+aa2)*q;
         a1 = 2*(aa0-aa2)*q;
         a2 = (aa0-aa1+aa2)*q;
@@ -219,16 +218,27 @@ public:
         b2 = (ab0-ab1+ab2)*q;
     }
     
+    /// set digital filter parameters directly
+    void set_bilinear_direct(double aa0, double aa1, double aa2, double ab1, double ab2)
+    {
+        a0 = aa0;
+        a1 = aa1;
+        a2 = aa2;
+        b1 = ab1;
+        b2 = ab2;
+    }
+    
+    
     /// RBJ peaking EQ
     /// @param freq   peak frequency
     /// @param q      q (correlated to freq/bandwidth, @see set_bp_rbj)
     /// @param peak   peak gain (1.0 means no peak, >1.0 means a peak, less than 1.0 is a dip)
-    inline void set_peakeq_rbj(float freq, float q, float peak, float sr)
+    inline void set_peakeq_rbj(double freq, double q, double peak, double sr)
     {
-        float A = sqrt(peak);
-        float w0 = freq * 2 * M_PI * (1.0 / sr);
-        float alpha = sin(w0) / (2 * q);
-        float ib0 = 1.0 / (1 + alpha/A);
+        double A = sqrt(peak);
+        double w0 = freq * 2 * M_PI * (1.0 / sr);
+        double alpha = sin(w0) / (2 * q);
+        double ib0 = 1.0 / (1 + alpha/A);
         a1 = b1 = -2*cos(w0) * ib0;
         a0 = ib0 * (1 + alpha*A);
         a2 = ib0 * (1 - alpha*A);
@@ -241,12 +251,12 @@ public:
     /// @param peak   shelf gain (1.0 means no peak, >1.0 means a peak, less than 1.0 is a dip)
     inline void set_lowshelf_rbj(float freq, float q, float peak, float sr)
     {
-        float A = sqrt(peak);
-        float w0 = freq * 2 * M_PI * (1.0 / sr);
-        float alpha = sin(w0) / (2 * q);
-        float cw0 = cos(w0);
-        float tmp = 2 * sqrt(A) * alpha;
-        float b0 = 0.f, ib0 = 0.f;
+        double A = sqrt(peak);
+        double w0 = freq * 2 * M_PI * (1.0 / sr);
+        double alpha = sin(w0) / (2 * q);
+        double cw0 = cos(w0);
+        double tmp = 2 * sqrt(A) * alpha;
+        double b0 = 0.f, ib0 = 0.f;
         
         a0 =    A*( (A+1) - (A-1)*cw0 + tmp);
         a1 =  2*A*( (A-1) - (A+1)*cw0);
@@ -269,12 +279,12 @@ public:
     /// @param peak   shelf gain (1.0 means no peak, >1.0 means a peak, less than 1.0 is a dip)
     inline void set_highshelf_rbj(float freq, float q, float peak, float sr)
     {
-        float A = sqrt(peak);
-        float w0 = freq * 2 * M_PI * (1.0 / sr);
-        float alpha = sin(w0) / (2 * q);
-        float cw0 = cos(w0);
-        float tmp = 2 * sqrt(A) * alpha;
-        float b0 = 0.f, ib0 = 0.f;
+        double A = sqrt(peak);
+        double w0 = freq * 2 * M_PI * (1.0 / sr);
+        double alpha = sin(w0) / (2 * q);
+        double cw0 = cos(w0);
+        double tmp = 2 * sqrt(A) * alpha;
+        double b0 = 0.f, ib0 = 0.f;
         
         a0 =    A*( (A+1) + (A-1)*cw0 + tmp);
         a1 = -2*A*( (A-1) + (A+1)*cw0);
@@ -292,8 +302,7 @@ public:
     }
     
     /// copy coefficients from another biquad
-    template<class U>
-    inline void copy_coeffs(const biquad_coeffs<U> &src)
+    inline void copy_coeffs(const biquad_coeffs &src)
     {
         a0 = src.a0;
         a1 = src.a1;
@@ -305,7 +314,7 @@ public:
     /// Return the filter's gain at frequency freq
     /// @param freq   Frequency to look up
     /// @param sr     Filter sample rate (used to convert frequency to angular frequency)
-    float freq_gain(float freq, float sr)
+    float freq_gain(float freq, float sr) const
     {
         typedef std::complex<double> cfloat;
         freq *= 2.0 * M_PI / sr;
@@ -316,7 +325,7 @@ public:
     
     /// Return H(z) the filter's gain at frequency freq
     /// @param z   Z variable (e^jw)
-    cfloat h_z(const cfloat &z)
+    cfloat h_z(const cfloat &z) const
     {
         
         return (cfloat(a0) + double(a1) * z + double(a2) * z*z) / (cfloat(1.0) + double(b1) * z + double(b2) * z*z);
@@ -329,31 +338,25 @@ public:
  * Uses "traditional" Direct I form (separate FIR and IIR halves).
  * don't use this for integers because it won't work
  */
-template<class Coeff = float, class T = float>
-struct biquad_d1: public biquad_coeffs<Coeff>
+struct biquad_d1: public biquad_coeffs
 {
-    using biquad_coeffs<Coeff>::a0;
-    using biquad_coeffs<Coeff>::a1;
-    using biquad_coeffs<Coeff>::a2;
-    using biquad_coeffs<Coeff>::b1;
-    using biquad_coeffs<Coeff>::b2;
     /// input[n-1]
-    T x1; 
+    double x1; 
     /// input[n-2]
-    T x2; 
+    double x2; 
     /// output[n-1]
-    T y1; 
+    double y1; 
     /// output[n-2]
-    T y2; 
+    double y2; 
     /// Constructor (initializes state to all zeros)
     biquad_d1()
     {
         reset();
     }
     /// direct I form with four state variables
-    inline T process(T in)
+    inline double process(double in)
     {
-        T out = in * a0 + x1 * a1 + x2 * a2 - y1 * b1 - y2 * b2;
+        double out = in * a0 + x1 * a1 + x2 * a2 - y1 * b1 - y2 * b2;
         x2 = x1;
         y2 = y1;
         x1 = in;
@@ -362,18 +365,28 @@ struct biquad_d1: public biquad_coeffs<Coeff>
     }
     
     /// direct I form with zero input
-    inline T process_zeroin()
+    inline double process_zeroin()
     {
-        T out = - y1 * b1 - y2 * b2;
+        double out = - y1 * b1 - y2 * b2;
         y2 = y1;
         y1 = out;
         return out;
     }
     
     /// simplified version for lowpass case with two zeros at -1
-    inline T process_lp(T in)
+    inline double process_lp(double in)
     {
-        T out = a0*(in + x1 + x1 + x2) - y1 * b1 - y2 * b2;
+        double out = a0*(in + x1 + x1 + x2) - y1 * b1 - y2 * b2;
+        x2 = x1;
+        y2 = y1;
+        x1 = in;
+        y1 = out;
+        return out;
+    }
+    /// simplified version for highpass case with two zeros at 1
+    inline double process_hp(double in)
+    {
+        double out = a0*(in - x1 - x1 + x2) - y1 * b1 - y2 * b2;
         x2 = x1;
         y2 = y1;
         x1 = in;
@@ -396,8 +409,8 @@ struct biquad_d1: public biquad_coeffs<Coeff>
         dsp::zero(x2);
         dsp::zero(y2);
     }
-    inline bool empty() {
-        return (y1 == 0.f && y2 == 0.f);
+    inline bool empty() const {
+        return (y1 == 0. && y2 == 0.);
     }
     
 };
@@ -409,28 +422,28 @@ struct biquad_d1: public biquad_coeffs<Coeff>
  * make more zipper noise than Direct I form, so it's better to
  * use it when filter coefficients are not changed mid-stream.
  */
-template<class Coeff = float, class T = float>
-struct biquad_d2: public biquad_coeffs<Coeff>
+struct biquad_d2: public biquad_coeffs
 {
-    using biquad_coeffs<Coeff>::a0;
-    using biquad_coeffs<Coeff>::a1;
-    using biquad_coeffs<Coeff>::a2;
-    using biquad_coeffs<Coeff>::b1;
-    using biquad_coeffs<Coeff>::b2;
     /// state[n-1]
-    float w1; 
+    double w1; 
     /// state[n-2]
-    float w2; 
+    double w2; 
     /// Constructor (initializes state to all zeros)
     biquad_d2()
     {
         reset();
     }
     /// direct II form with two state variables
-    inline T process(T in)
+    inline double process(double in)
     {
-        T tmp = in - w1 * b1 - w2 * b2;
-        T out = tmp * a0 + w1 * a1 + w2 * a2;
+        double n = in;
+        dsp::sanitize_denormal(n);
+        dsp::sanitize(n);
+        dsp::sanitize(w1);
+        dsp::sanitize(w2);
+
+        double tmp = n - w1 * b1 - w2 * b2;
+        double out = tmp * a0 + w1 * a1 + w2 * a2;
         w2 = w1;
         w1 = tmp;
         return out;
@@ -438,17 +451,17 @@ struct biquad_d2: public biquad_coeffs<Coeff>
     
     // direct II form with two state variables, lowpass version
     // interesting fact: this is actually slower than the general version!
-    inline T process_lp(T in)
+    inline double process_lp(double in)
     {
-        T tmp = in - w1 * b1 - w2 * b2;
-        T out = (tmp  + w2 + w1* 2) * a0;
+        double tmp = in - w1 * b1 - w2 * b2;
+        double out = (tmp  + w2 + w1* 2) * a0;
         w2 = w1;
         w1 = tmp;
         return out;
     }
 
     /// Is the filter state completely silent? (i.e. set to 0 by sanitize function)
-    inline bool empty() {
+    inline bool empty() const {
         return (w1 == 0.f && w2 == 0.f);
     }
     
@@ -473,31 +486,25 @@ struct biquad_d2: public biquad_coeffs<Coeff>
  * Uses "traditional" Direct I form (separate FIR and IIR halves).
  * don't use this for integers because it won't work
  */
-template<class Coeff = float, class T = float>
-struct biquad_d1_lerp: public biquad_coeffs<Coeff>
+struct biquad_d1_lerp: public biquad_coeffs
 {
-    using biquad_coeffs<Coeff>::a0;
-    using biquad_coeffs<Coeff>::a1;
-    using biquad_coeffs<Coeff>::a2;
-    using biquad_coeffs<Coeff>::b1;
-    using biquad_coeffs<Coeff>::b2;
-    Coeff a0cur, a1cur, a2cur, b1cur, b2cur;
-    Coeff a0delta, a1delta, a2delta, b1delta, b2delta;
+    double a0cur, a1cur, a2cur, b1cur, b2cur;
+    double a0delta, a1delta, a2delta, b1delta, b2delta;
     /// input[n-1]
-    T x1; 
+    double  x1; 
     /// input[n-2]
-    T x2; 
+    double  x2; 
     /// output[n-1]
-    T y1; 
+    double  y1; 
     /// output[n-2]
-    T y2; 
+    double  y2;
     /// Constructor (initializes state to all zeros)
     biquad_d1_lerp()
     {
         reset();
     }
     #define _DO_COEFF(coeff) coeff##delta = (coeff - coeff##cur) * (frac)
-    void big_step(Coeff frac)
+    void big_step(double frac)
     {
         _DO_COEFF(a0);
         _DO_COEFF(a1);
@@ -507,9 +514,9 @@ struct biquad_d1_lerp: public biquad_coeffs<Coeff>
     }
     #undef _DO_COEFF
     /// direct I form with four state variables
-    inline T process(T in)
+    inline double process(double in)
     {
-        T out = in * a0cur + x1 * a1cur + x2 * a2cur - y1 * b1cur - y2 * b2cur;
+        double out = in * a0cur + x1 * a1cur + x2 * a2cur - y1 * b1cur - y2 * b2cur;
         x2 = x1;
         y2 = y1;
         x1 = in;
@@ -523,9 +530,9 @@ struct biquad_d1_lerp: public biquad_coeffs<Coeff>
     }
     
     /// direct I form with zero input
-    inline T process_zeroin()
+    inline double process_zeroin()
     {
-        T out = - y1 * b1 - y2 * b2;
+        double  out = - y1 * b1 - y2 * b2;
         y2 = y1;
         y1 = out;
         b1cur += b1delta;
@@ -534,9 +541,9 @@ struct biquad_d1_lerp: public biquad_coeffs<Coeff>
     }
     
     /// simplified version for lowpass case with two zeros at -1
-    inline T process_lp(T in)
+    inline double process_lp(double in)
     {
-        T out = a0*(in + x1 + x1 + x2) - y1 * b1 - y2 * b2;
+        double out = a0*(in + x1 + x1 + x2) - y1 * b1 - y2 * b2;
         x2 = x1;
         y2 = y1;
         x1 = in;
@@ -570,11 +577,79 @@ struct biquad_d1_lerp: public biquad_coeffs<Coeff>
         dsp::zero(b2cur);
     }
     inline bool empty() {
-        return (y1 == 0.f && y2 == 0.f);
+        return (y1 == 0. && y2 == 0.);
     }
     
 };
     
+/// Compose two filters in series
+template<class F1, class F2>
+class filter_compose {
+public:
+    typedef std::complex<float> cfloat;
+    F1 f1;
+    F2 f2;
+public:
+    float process(float value) {
+        return f2.process(f1.process(value));
+    }
+    
+    inline cfloat h_z(const cfloat &z) const {
+        return f1.h_z(z) * f2.h_z(z);
+    }
+    
+    /// Return the filter's gain at frequency freq
+    /// @param freq   Frequency to look up
+    /// @param sr     Filter sample rate (used to convert frequency to angular frequency)
+    float freq_gain(float freq, float sr) const
+    {
+        typedef std::complex<double> cfloat;
+        freq *= 2.0 * M_PI / sr;
+        cfloat z = 1.0 / exp(cfloat(0.0, freq));
+        
+        return std::abs(h_z(z));
+    }
+    
+    void sanitize() {
+        f1.sanitize();
+        f2.sanitize();
+    }
+};
+
+/// Compose two filters in parallel
+template<class F1, class F2>
+class filter_sum {
+public:
+    typedef std::complex<double> cfloat;
+    F1 f1;
+    F2 f2;
+public:
+    double process(double value) {
+        return f2.process(value) + f1.process(value);
+    }
+    
+    inline cfloat h_z(const cfloat &z) const {
+        return f1.h_z(z) + f2.h_z(z);
+    }
+    
+    /// Return the filter's gain at frequency freq
+    /// @param freq   Frequency to look up
+    /// @param sr     Filter sample rate (used to convert frequency to angular frequency)
+    float freq_gain(float freq, float sr) const
+    {
+        typedef std::complex<double> cfloat;
+        freq *= 2.0 * M_PI / sr;
+        cfloat z = 1.0 / exp(cfloat(0.0, freq));
+        
+        return std::abs(h_z(z));
+    }
+    
+    void sanitize() {
+        f1.sanitize();
+        f2.sanitize();
+    }
+};
+
 };
 
 #endif
