@@ -30,6 +30,7 @@
 #include "envelope.h"
 #include "modmatrix.h"
 #include "metadata.h"
+#include "giface.h"
 
 namespace calf_plugins {
 
@@ -42,9 +43,10 @@ class monosynth_audio_module: public audio_module<monosynth_metadata>, public li
 public:
     uint32_t srate, crate;
     static dsp::waveform_family<MONOSYNTH_WAVE_BITS> *waves;
-    dsp::waveform_oscillator<MONOSYNTH_WAVE_BITS> osc1, osc2;
+    dsp::waveform_oscillator<MONOSYNTH_WAVE_BITS> osc1, osc2, detosc;
     dsp::triangle_lfo lfo1, lfo2;
-    dsp::biquad_d1_lerp<float> filter, filter2;
+    dsp::simple_oscillator unison_osc;
+    dsp::biquad_d1_lerp filter, filter2;
     /// The step code is producing non-zero values
     bool running;
     /// This is the last non-zero buffer (set on calculate_step after fadeout is complete, the next calculate_step will zero running)
@@ -73,7 +75,7 @@ public:
     /// Filter type on the last calculate_step
     int last_filter_type;
     float freq, start_freq, target_freq, cutoff, fgain, fgain_delta, separation;
-    float detune, xpose, xfade, ampctl, fltctl;
+    float detune, xpose1, xpose2, xfade, ampctl, fltctl;
     float odcr, porta_time, lfo_bend;
     /// Modulation wheel position (0.f-1.f)
     float modwheel_value;
@@ -116,11 +118,12 @@ public:
     float velocity;
     /// Last value of oscillator mix ratio
     float last_xfade;
+    /// Last value of unison amount
+    float last_unison;
     /// Current calculated mod matrix outputs
     float moddest[moddest_count];
      
     monosynth_audio_module();    
-    static void precalculate_waves(progress_report_iface *reporter);
     void set_sample_rate(uint32_t sr);
     void delayed_note_on();
     /// Release a note (physically), called from note-off handler or when note-off has been scheduled after note-on (very short queued note)
@@ -145,7 +148,7 @@ public:
     void params_changed();
     void activate();
     void deactivate();
-    void post_instantiate()
+    void post_instantiate(uint32_t)
     {
         precalculate_waves(progress_report);
     }
@@ -160,27 +163,31 @@ public:
     /// Run two filters (one per channel) to produce stereo output samples.
     void calculate_buffer_stereo();
     /// Retrieve filter graph (which is 'live' so it cannot be generated by get_static_graph), or fall back to get_static_graph.
-    bool get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const;
+    bool get_layers(int index, int generation, unsigned int &layers) const { layers = LG_REALTIME_GRAPH; return true; }
     /// @retval true if the filter 1 is to be used for the left channel and filter 2 for the right channel
-    /// @retval false if filters are to be connected in series and sent (mono) to both channels    
-    inline bool is_stereo_filter() const
-    {
-        return filter_type == flt_2lp12 || filter_type == flt_2bp6;
-    }
     /// No CV inputs for now
     bool is_cv(int param_no) const { return false; }
     /// Practically all the stuff here is noisy
     bool is_noisy(int param_no) const { return param_no != par_cutoff; }
-    /// Calculate control signals and produce step_size samples of output.
-    void calculate_step();
-    /// Apply anti-click'n'pop fadeout (used at the end of the sound)
-    void apply_fadeout();
     /// Main processing function
     uint32_t process(uint32_t offset, uint32_t nsamples, uint32_t inputs_mask, uint32_t outputs_mask);
     /// Send all configure variables set within a plugin to given destination (which may be limited to only those that plugin understands)
     virtual void send_configures(send_configure_iface *sci) { return mod_matrix_impl::send_configures(sci); }
     virtual char *configure(const char *key, const char *value) { return mod_matrix_impl::configure(key, value); }
+private:
+    void reset();
     float get_lfo(dsp::triangle_lfo &lfo, int param);
+    /// Apply anti-click'n'pop fadeout (used at the end of the sound)
+    void apply_fadeout();
+    /// Calculate control signals and produce step_size samples of output.
+    void calculate_step();
+    /// @retval false if filters are to be connected in series and sent (mono) to both channels    
+    inline bool is_stereo_filter() const
+    {
+        return filter_type == flt_2lp12 || filter_type == flt_2bp6;
+    }
+    static void precalculate_waves(progress_report_iface *reporter);
 };
 
 };
