@@ -21,12 +21,12 @@
 
 #include <calf/gtk_session_env.h>
 #include <calf/gtk_main_win.h>
+#include <map>
 
 using namespace calf_plugins;
 
 void gtk_session_environment::init_gui(int &argc, char **&argv)
 {
-    gtk_rc_add_default_file(PKGLIBDIR "calf.rc");
     gtk_init(&argc, &argv);
 }
 
