@@ -18,24 +18,7 @@
  * Boston, MA  02110-1301  USA
  */
  
-#include <config.h>
-#include <assert.h>
-#include <sys/time.h>
-#include <calf/ctl_curve.h>
-#include <calf/ctl_keyboard.h>
-#include <calf/ctl_knob.h>
-#include <calf/ctl_led.h>
-#include <calf/ctl_tube.h>
-#include <calf/ctl_vumeter.h>
-#include <calf/custom_ctl.h>
-#include <calf/giface.h>
-#include <calf/gui.h>
 #include <calf/gui_controls.h>
-#include <calf/utils.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
-#include <calf/ctl_linegraph.h>
-#include <iostream>
 
 using namespace calf_plugins;
 using namespace calf_utils;
@@ -83,6 +66,27 @@ float control_base::get_float(const char *name, float def_value)
     return value;
 }
 
+std::vector<double> control_base::get_vector(const char *name, std::string &value)
+{
+    std::vector<double> t;
+    
+    if (attribs.count(name)) {
+        value = attribs[name];
+    }
+        
+    string::size_type lpos = value.find_first_not_of(" ", 0);
+    string::size_type pos  = value.find_first_of(" ", lpos);
+    while (string::npos != pos || string::npos != lpos) {
+        double val;
+        stringstream stream(value.substr(lpos, pos - lpos).c_str());
+        stream >> val;
+        t.push_back(val);
+        lpos = value.find_first_not_of(" ", pos);
+        pos  = value.find_first_of(" ", lpos);
+    }
+    return t;
+}
+
 void control_base::set_visibilty(bool state)
 {
     if (state) {
@@ -252,7 +256,7 @@ void param_control::create_value_entry(GtkWidget *widget, int x, int y)
     gtk_window_set_decorated (GTK_WINDOW(entrywin), FALSE);
     gtk_window_set_skip_taskbar_hint (GTK_WINDOW(entrywin), TRUE);
     gtk_window_set_skip_pager_hint (GTK_WINDOW(entrywin), TRUE);
-    gtk_window_set_transient_for (GTK_WINDOW(entrywin), GTK_WINDOW (gui->window->toplevel));
+    gtk_window_set_transient_for (GTK_WINDOW(entrywin), GTK_WINDOW (gtk_widget_get_toplevel(gui->window->toplevel)));
     gtk_window_set_gravity(GTK_WINDOW(entrywin), GDK_GRAVITY_CENTER);
     gtk_widget_set_events (GTK_WIDGET(entrywin), GDK_FOCUS_CHANGE_MASK);
     g_signal_connect (G_OBJECT(entrywin), "focus-out-event", G_CALLBACK (value_entry_unfocus), this);
@@ -290,9 +294,14 @@ GtkWidget *combo_box_param_control::create(plugin_gui *_gui, int _param_no)
         for (int j = (int)props.min; j <= (int)props.max; j++)
             gtk_list_store_insert_with_values (lstore, NULL, j - (int)props.min, 0, props.choices[j - (int)props.min], 1, calf_utils::i2s(j).c_str(), -1);
     }
+    
+    // set pixbuf
+    calf_combobox_set_arrow(CALF_COMBOBOX(widget),
+        gui->window->get_environment()->get_image_factory()->get("combo_arrow"));
+    
     gtk_combo_box_set_model (GTK_COMBO_BOX(widget), GTK_TREE_MODEL(lstore));
     g_signal_connect (GTK_OBJECT (widget), "changed", G_CALLBACK (combo_value_changed), (gpointer)this);
-    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-Combobox");
+    gtk_widget_set_name(widget, "Calf-Combobox");
     return widget;
 }
 
@@ -303,6 +312,7 @@ void combo_box_param_control::set()
     {
         const parameter_properties &props = get_props();
         gtk_combo_box_set_active (GTK_COMBO_BOX (widget), (int)gui->plugin->get_param_value(param_no) - (int)props.min);
+        gtk_widget_queue_draw(widget);
     }
 }
 
@@ -434,6 +444,13 @@ GtkWidget *hscale_param_control::create(plugin_gui *_gui, int _param_no)
         gtk_range_set_inverted(GTK_RANGE(widget), TRUE);
     }
     int size = get_int("size", 2);
+    
+    // set pixbuf
+    image_factory *images = gui->window->get_environment()->get_image_factory();
+    char iname[64];
+    sprintf(iname, "slider_%d_horiz", size);
+    calf_fader_set_pixbuf(CALF_FADER(widget), images->get(iname));
+    
     char *name = g_strdup_printf("Calf-HScale%i", size);
     gtk_widget_set_name(GTK_WIDGET(widget), name);
     gtk_widget_set_size_request (widget, size * 100, -1);
@@ -441,11 +458,12 @@ GtkWidget *hscale_param_control::create(plugin_gui *_gui, int _param_no)
 
     if (attribs.count("width"))
         gtk_widget_set_size_request (widget, get_int("width", 200), -1);
-    if (attribs.count("position"))
-    {
+    if (attribs.count("position")) {
         string v = attribs["position"];
         if (v == "top") gtk_scale_set_value_pos(GTK_SCALE(widget), GTK_POS_TOP);
         if (v == "bottom") gtk_scale_set_value_pos(GTK_SCALE(widget), GTK_POS_BOTTOM);
+        if (v == "left") gtk_scale_set_value_pos(GTK_SCALE(widget), GTK_POS_LEFT);
+        if (v == "right") gtk_scale_set_value_pos(GTK_SCALE(widget), GTK_POS_RIGHT);
     }
     return widget;
 }
@@ -498,6 +516,13 @@ GtkWidget *vscale_param_control::create(plugin_gui *_gui, int _param_no)
         gtk_range_set_inverted(GTK_RANGE(widget), TRUE);
     }
     int size = get_int("size", 2);
+    
+    // set pixbuf
+    image_factory *images = gui->window->get_environment()->get_image_factory();
+    char iname[64];
+    sprintf(iname, "slider_%d_vert", size);
+    calf_fader_set_pixbuf(CALF_FADER(widget), images->get(iname));
+    
     char *name = g_strdup_printf("Calf-VScale%i", size);
     gtk_widget_set_size_request (widget, -1, size * 100);
     gtk_widget_set_name(GTK_WIDGET(widget), name);
@@ -603,13 +628,14 @@ GtkWidget *vumeter_param_control::create(plugin_gui *_gui, int _param_no)
     gui = _gui, param_no = _param_no;
     // const parameter_properties &props = get_props();
     widget = calf_vumeter_new ();
+    CalfVUMeter *vu = CALF_VUMETER(widget);
     gtk_widget_set_name(GTK_WIDGET(widget), "calf-vumeter");
-    calf_vumeter_set_mode (CALF_VUMETER (widget), (CalfVUMeterMode)get_int("mode", 0));
-    CALF_VUMETER(widget)->vumeter_hold = get_float("hold", 0);
-    CALF_VUMETER(widget)->vumeter_falloff = get_float("falloff", 0.f);
-    CALF_VUMETER(widget)->vumeter_width = get_int("width", 80);
-    CALF_VUMETER(widget)->vumeter_height = get_int("height", 18);
-    CALF_VUMETER(widget)->vumeter_position = get_int("position", 0);
+    calf_vumeter_set_mode (vu, (CalfVUMeterMode)get_int("mode", 0));
+    vu->vumeter_hold = get_float("hold", 0);
+    vu->vumeter_falloff = get_float("falloff", 0.f);
+    vu->vumeter_width = get_int("width", 80);
+    vu->vumeter_height = get_int("height", 18);
+    vu->vumeter_position = get_int("position", 0);
     gtk_widget_set_name(GTK_WIDGET(widget), "Calf-VUMeter");
     return widget;
 }
@@ -648,11 +674,12 @@ GtkWidget *tube_param_control::create(plugin_gui *_gui, int _param_no)
 {
     gui = _gui, param_no = _param_no;
     // const parameter_properties &props = get_props();
-    widget = calf_tube_new ();
-    gtk_widget_set_name(GTK_WIDGET(widget), "calf-tube");
-    CALF_TUBE(widget)->size = get_int("size", 2);
-    CALF_TUBE(widget)->direction = get_int("direction", 2);
-    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-Tube");
+    GtkWidget *widget = calf_tube_new ();
+    CalfTube *tube = CALF_TUBE(widget);
+    gtk_widget_set_name(widget, "calf-tube");
+    tube->size = get_int("size", 2);
+    tube->direction = get_int("direction", 2);
+    gtk_widget_set_name(widget, "Calf-Tube");
     return widget;
 }
 
@@ -832,21 +859,41 @@ GtkWidget *knob_param_control::create(plugin_gui *_gui, int _param_no)
     gui = _gui;
     param_no = _param_no;
     const parameter_properties &props = get_props();
-    
-    //widget = calf_knob_new_with_range (props.to_01 (gui->plugin->get_param_value(param_no)), 0, 1, 0.01);
     widget = calf_knob_new();
+    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-Knob");
+    CalfKnob * knob = CALF_KNOB(widget);
+    
     float increment = props.get_increment();
     gtk_range_get_adjustment(GTK_RANGE(widget))->step_increment = increment;
-    CALF_KNOB(widget)->default_value = props.to_01(props.def_value);
-    CALF_KNOB(widget)->knob_type = get_int("type");
-    CALF_KNOB(widget)->knob_size = get_int("size", 2);
-    if(CALF_KNOB(widget)->knob_size > 5) {
-        CALF_KNOB(widget)->knob_size = 5;
-    } else if (CALF_KNOB(widget)->knob_size < 1) {
-        CALF_KNOB(widget)->knob_size = 1;
+    
+    knob->default_value = props.to_01(props.def_value);
+    knob->type = get_int("type");
+    calf_knob_set_size(knob, get_int("size", 2));
+    
+    // set pixbuf
+    char imgname[16];
+    sprintf(imgname, "knob_%d", get_int("size", 2));
+    calf_knob_set_pixbuf(knob, gui->window->get_environment()->get_image_factory()->get(imgname));
+    
+    //char ticks[128];
+    std::ostringstream ticks_;
+    //std::string str = ticks.str();
+    double min = double(props.min);
+    double max = double(props.max);
+    switch (knob->type) {
+        default:
+        case 0: ticks_ << min << " " << max; break;
+        case 1: ticks_ << min << " " << props.from_01(0.5) << " " << max; break;
+        case 2: ticks_ << min << " " << max; break;
+        case 3: ticks_ << min << " " << props.from_01(0.25) << " " << props.from_01(0.5) << " " << props.from_01(0.75) << " " << max; break;
     }
+    std::string ticks = ticks_.str();
+    vector<double> t = get_vector("ticks", ticks);
+    std::sort(t.begin(), t.end());
+    for (unsigned int i = 0; i < t.size(); i++)
+        t[i] = props.to_01(t[i]);
+    knob->ticks = t;
     g_signal_connect(GTK_OBJECT(widget), "value-changed", G_CALLBACK(knob_value_changed), (gpointer)this);
-    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-Knob");
     return widget;
 }
 
@@ -877,11 +924,22 @@ GtkWidget *toggle_param_control::create(plugin_gui *_gui, int _param_no)
     gui = _gui;
     param_no = _param_no;
     widget  = calf_toggle_new ();
+    CalfToggle * toggle = CALF_TOGGLE(widget);
+    calf_toggle_set_size(toggle, get_int("size", 2));
     
-    CALF_TOGGLE(widget)->size = get_int("size", 2);
+    // set pixbuf
+    image_factory *images = gui->window->get_environment()->get_image_factory();
+    char imgname[64];
+    if (attribs.count("icon") != 0) {
+        sprintf(imgname, "toggle_%d_%s", get_int("size", 2), attribs["icon"].c_str());
+        if (!images->available(imgname))
+            sprintf(imgname, "toggle_%d", get_int("size", 2));
+    } else
+        sprintf(imgname, "toggle_%d", get_int("size", 2));
+    calf_toggle_set_pixbuf(toggle, images->get(imgname));
     
     g_signal_connect (GTK_OBJECT (widget), "value-changed", G_CALLBACK (toggle_value_changed), (gpointer)this);
-    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-ToggleButton");
+    //gtk_widget_set_name(GTK_WIDGET(widget), "Calf-ToggleButton");
     return widget;
 }
 
@@ -896,7 +954,8 @@ void toggle_param_control::set()
 {
     _GUARD_CHANGE_
     const parameter_properties &props = get_props();
-    gtk_range_set_value(GTK_RANGE(widget), props.to_01 (gui->plugin->get_param_value(param_no)));
+    float value = gui->plugin->get_param_value(param_no);
+    gtk_range_set_value(GTK_RANGE(widget), props.to_01(value));
 }
 
 void toggle_param_control::toggle_value_changed(GtkWidget *widget, gpointer value)
@@ -912,10 +971,15 @@ GtkWidget *tap_button_param_control::create(plugin_gui *_gui, int _param_no)
     gui       = _gui;
     param_no  = _param_no;
     last_time = 0;
-    init_time = 0;
     avg_value = 0;
     value     = 0;
+    timer     = 0;
     widget    = calf_tap_button_new ();
+    // set pixbuf
+    calf_tap_button_set_pixbufs(CALF_TAP_BUTTON(widget),
+        gui->window->get_environment()->get_image_factory()->get("tap_inactive"),
+        gui->window->get_environment()->get_image_factory()->get("tap_prelight"),
+        gui->window->get_environment()->get_image_factory()->get("tap_active"));
     //CALF_TAP(widget)->size = get_int("size", 2);
     g_signal_connect (GTK_OBJECT (widget), "button-press-event", G_CALLBACK (tap_button_pressed), (gpointer)this);
     g_signal_connect (GTK_OBJECT (widget), "released", G_CALLBACK (tap_button_released), (gpointer)this);
@@ -932,19 +996,6 @@ void tap_button_param_control::get()
 void tap_button_param_control::set()
 {
     _GUARD_CHANGE_
-    if(last_time) {
-        timeval tv;
-        gettimeofday(&tv, 0);
-        unsigned long _now = tv.tv_sec * 1000;
-        if(_now > init_time + 2000) {
-            // user stopped tapping
-            avg_value = 0;
-            last_time = 0;
-            init_time = 0;
-            CALF_TAP_BUTTON(widget)->state = 0;
-            gtk_widget_queue_draw(widget);
-        }
-    }
 }
 
 gboolean tap_button_param_control::tap_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer value)
@@ -953,14 +1004,9 @@ gboolean tap_button_param_control::tap_button_pressed(GtkWidget *widget, GdkEven
     CalfTapButton *tap = CALF_TAP_BUTTON(widget);
     
     guint time = 0;
-    if (event->type == GDK_BUTTON_PRESS and event->button == 1)
-    {
-        timeval tv;
-        gettimeofday(&tv, 0);
-        ctl->init_time = tv.tv_sec * 1000;
+    if (event->type == GDK_BUTTON_PRESS and event->button == 1) {
         time = event->time;
         tap->state = 2;
-        
         if(ctl->last_time) {
             if(ctl->avg_value)
                 ctl->avg_value = (ctl->avg_value * 3 + (time - ctl->last_time)) / 4.f;
@@ -972,10 +1018,26 @@ gboolean tap_button_param_control::tap_button_pressed(GtkWidget *widget, GdkEven
                 ctl->get();
         }
         ctl->last_time = time;
+        if (ctl->timer)
+            gtk_timeout_remove(ctl->timer);
+        ctl->timer = gtk_timeout_add(2000, (GtkFunction)tap_button_stop_waiting, (gpointer)ctl);
         gtk_widget_queue_draw(widget);
     }
     return FALSE;
 }
+void tap_button_param_control::tap_button_stop_waiting(gpointer data)
+{
+    tap_button_param_control *ctl = (tap_button_param_control *)data;
+    if (ctl->timer) {
+        ctl->avg_value = 0;
+        ctl->last_time = 0;
+        CALF_TAP_BUTTON(ctl->widget)->state = 0;
+        gtk_widget_queue_draw(ctl->widget);
+        gtk_timeout_remove(ctl->timer);
+        ctl->timer = 0;
+        gtk_widget_queue_draw(ctl->widget);
+    }
+}
 gboolean tap_button_param_control::tap_button_released(GtkWidget *widget, gpointer value)
 {
     tap_button_param_control *ctl = (tap_button_param_control *)value;
@@ -1062,6 +1124,23 @@ void curve_param_control::send_configure(const char *key, const char *value)
     }
 }
 
+/******************************** Meter Scale ********************************/
+
+GtkWidget *meter_scale_param_control::create(plugin_gui *_gui, int _param_no)
+{
+    gui = _gui;
+    param_no = _param_no;
+    widget  = calf_meter_scale_new ();
+    CalfMeterScale *ms = CALF_METER_SCALE(widget);
+    gtk_widget_set_name(widget, "Calf-MeterScale");
+    string str   = "0 0.5 1";
+    ms->marker   = get_vector("marker", str);
+    ms->mode     = (CalfVUMeterMode)get_int("mode", 0);
+    ms->position = get_int("position", 0);
+    ms->dots     = get_int("dots", 0);
+    return widget;
+}
+
 /******************************** Entry ********************************/
 
 GtkWidget *entry_param_control::create(plugin_gui *_gui, int _param_no)
@@ -1109,9 +1188,10 @@ GtkWidget *filechooser_param_control::create(plugin_gui *_gui, int _param_no)
     g_signal_connect(GTK_OBJECT(widget), "file-set", G_CALLBACK(filechooser_value_changed), (gpointer)this);
     if (attribs.count("width"))
         gtk_widget_set_size_request (widget, get_int("width", 200), -1);
-    if (attribs.count("width_chars"))
-         gtk_file_chooser_button_set_width_chars (filechooser, get_int("width_chars"));
-         gtk_widget_set_name(GTK_WIDGET(widget), "Calf-FileButton");
+    if (attribs.count("width_chars")) {
+        gtk_file_chooser_button_set_width_chars (filechooser, get_int("width_chars"));
+        gtk_widget_set_name(GTK_WIDGET(widget), "Calf-FileButton");
+    }
     return widget;
 }
 
@@ -1173,8 +1253,6 @@ GtkWidget *line_graph_param_control::create(plugin_gui *a_gui, int a_param_no)
     
     widget                     = calf_line_graph_new ();
     
-    gtk_widget_set_name(GTK_WIDGET(widget), "calf-graph");
-    
     CalfLineGraph *clg         = CALF_LINE_GRAPH(widget);
     widget->requisition.width  = get_int("width", 40);
     widget->requisition.height = get_int("height", 40);
@@ -1237,10 +1315,10 @@ GtkWidget *line_graph_param_control::create(plugin_gui *a_gui, int a_param_no)
             if(param_z_name != "") {
                 int param_z_no = gui->get_param_no_by_name(param_z_name);
                 const parameter_properties &handle_z_props = *gui->plugin->get_metadata_iface()->get_param_props(param_z_no);
-                handle->dimensions = 3;
                 handle->param_z_no = param_z_no;
                 handle->value_z = handle_z_props.to_01(gui->plugin->get_param_value(param_z_no));
                 handle->default_value_z = handle_z_props.to_01(handle_z_props.def_value);
+                handle->props_z = handle_z_props;
             } else {
                 handle->param_z_no = -1;
             }
@@ -1300,7 +1378,7 @@ void line_graph_param_control::get()
         } else if(clg->handle_hovered >= 0) {
             FreqHandle *handle = &clg->freq_handles[clg->handle_hovered];
 
-            if(handle->dimensions == 3) {
+            if(handle->param_z_no > -1) {
                 const parameter_properties &handle_z_props = *gui->plugin->get_metadata_iface()->get_param_props(handle->param_z_no);
                 float value_z = handle_z_props.from_01(handle->value_z);
                 gui->set_param_value(handle->param_z_no, value_z, this);
@@ -1360,7 +1438,7 @@ void line_graph_param_control::set()
                 }
             }
 
-            if(handle->dimensions == 3 && handle->param_z_no >= 0) {
+            if(handle->param_z_no >= 0) {
                 const parameter_properties &handle_z_props = *gui->plugin->get_metadata_iface()->get_param_props(handle->param_z_no);
                 float value_z = gui->plugin->get_param_value(handle->param_z_no);
                 handle->value_z = handle_z_props.to_01(value_z);
@@ -1411,7 +1489,6 @@ GtkWidget *phase_graph_param_control::create(plugin_gui *_gui, int _param_no)
     gui = _gui;
     param_no = _param_no;
     widget = calf_phase_graph_new ();
-    gtk_widget_set_name(GTK_WIDGET(widget), "calf-phase");
     CalfPhaseGraph *clg = CALF_PHASE_GRAPH(widget);
     widget->requisition.width = get_int("size", 40);
     widget->requisition.height = get_int("size", 40);
@@ -1434,6 +1511,131 @@ phase_graph_param_control::~phase_graph_param_control()
 {
 }
 
+
+/******************************** Tuner ********************************/
+
+void tuner_param_control::on_idle()
+{
+    if (get_int("refresh", 0))
+        set();
+}
+
+GtkWidget *tuner_param_control::create(plugin_gui *_gui, int _param_no)
+{
+    gui = _gui;
+    param_no = _param_no;
+    widget = calf_tuner_new ();
+    //CalfTuner *tuner = CALF_TUNER(widget);
+    widget->requisition.width = get_int("width", 40);
+    widget->requisition.height = get_int("height", 40);
+    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-Tuner");
+    
+    const string &cents_name = attribs["param_cents"];
+    if (cents_name != "")
+        cents_no = gui->get_param_no_by_name(cents_name);
+    else
+        cents_no = 0;
+    return widget;
+}
+
+void tuner_param_control::set()
+{
+    _GUARD_CHANGE_
+    GtkWidget *tw = gtk_widget_get_toplevel(widget);
+    CalfTuner *tuner = CALF_TUNER(widget);
+    tuner->note = gui->plugin->get_param_value(param_no);
+    tuner->cents = gui->plugin->get_param_value(cents_no);
+    if (tw && GTK_WIDGET_TOPLEVEL(tw) && widget->window) {
+        gtk_widget_queue_draw(widget);
+    }
+}
+
+tuner_param_control::~tuner_param_control()
+{
+}
+
+/******************************** Pattern ********************************/
+
+GtkWidget *pattern_param_control::create(plugin_gui *_gui, int _param_no)
+{
+    gui = _gui;
+    param_no = _param_no;
+    widget = calf_pattern_new ();
+    widget->requisition.width = get_int("width", 300);
+    widget->requisition.height = get_int("height", 60);
+    const string &beats_name = attribs["beats"];
+    if (beats_name != "") {
+        param_beats = gui->get_param_no_by_name(beats_name);
+        gui->add_param_ctl(param_beats, this);
+    } else param_beats = -1;
+    const string &bars_name = attribs["bars"];
+    if (bars_name != "") {
+        param_bars = gui->get_param_no_by_name(bars_name);
+        gui->add_param_ctl(param_bars, this);
+    } else param_bars = -1;
+    gtk_widget_set_name(GTK_WIDGET(widget), "Calf-Pattern");
+    g_signal_connect(GTK_OBJECT(widget), "handle-changed", (GCallback)on_handle_changed, this);
+    return widget;
+}
+
+void pattern_param_control::set()
+{
+    _GUARD_CHANGE_
+    CalfPattern *p = CALF_PATTERN(widget);
+
+    int b;
+    if (param_beats >= 0) {
+        b = gui->plugin->get_param_value(param_beats);
+        if (b != p->beats) {
+            p->beats = b;
+            p->force_redraw = true;
+            gtk_widget_queue_draw(widget);
+        }
+    }
+    if (param_bars >= 0) {
+        b = gui->plugin->get_param_value(param_bars);
+        if (b != p->bars) {
+            p->bars = b;
+            p->force_redraw = true;
+            gtk_widget_queue_draw(widget);
+        }
+    }
+}
+
+void pattern_param_control::send_configure(const char *key, const char *value)
+{
+    string orig_key = attribs["key"];
+    if (orig_key != key)
+        return;
+
+    CalfPattern *p = CALF_PATTERN(widget);
+    stringstream ss(value);
+    _GUARD_CHANGE_
+    for (int i = 0; i < p->bars; i++) {
+        for (int j = 0; j < p->beats; j++) {
+            ss >> p->values[i][j];
+        }
+    }
+    p->force_redraw = true;
+    gtk_widget_queue_draw(widget);
+}
+
+void pattern_param_control::on_handle_changed(CalfPattern *widget, calf_pattern_handle *handle, pattern_param_control *pThis)
+{
+    CalfPattern *p = CALF_PATTERN(widget);
+    stringstream ss;
+    for (int i = 0; i < p->bars; i++) {
+        for (int j = 0; j < p->beats; j++) {
+            ss << p->values[i][j] << " ";
+        }
+    }
+    assert(pThis);
+    string key = pThis->attribs["key"];
+    const char *error_or_null = pThis->gui->plugin->configure(key.c_str(), ss.str().c_str());
+    if (error_or_null)
+        g_warning("Unexpected error: %s", error_or_null);
+}
+
 /******************************** List View ********************************/
 
 GtkWidget *listview_param_control::create(plugin_gui *_gui, int _param_no)
@@ -1563,7 +1765,7 @@ void listview_param_control::on_edited(GtkCellRenderer *renderer, gchar *path, g
     }
     else
     {
-        GtkWidget *dialog = gtk_message_dialog_new(pThis->gui->window->toplevel, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+        GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(pThis->gui->window->toplevel), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
             "%s", error.c_str());
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
@@ -1589,14 +1791,19 @@ GtkWidget *notebook_param_control::create(plugin_gui *_gui, int _param_no)
         page = gui->plugin->get_param_value(param_no);
     GtkWidget *nb = calf_notebook_new();
     widget = GTK_WIDGET(nb);
-    gtk_widget_set_name(GTK_WIDGET(nb), "Calf-Notebook");
+    calf_notebook_set_pixbuf(CALF_NOTEBOOK(nb),
+        gui->window->get_environment()->get_image_factory()->get("notebook_screw"));
+    gtk_widget_set_name(widget, "Calf-Notebook");
     gtk_notebook_set_current_page(GTK_NOTEBOOK(widget), page);
     return nb;
 }
 void notebook_param_control::created()
 {
+    hook_params();
+    gtk_widget_show_all(widget);
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(widget), page);
     g_signal_connect (GTK_OBJECT (widget), "switch-page", G_CALLBACK (notebook_page_changed), (gpointer)this);
-    set();
+    //set();
 }
 void notebook_param_control::get()
 {
