@@ -24,6 +24,15 @@
 #include <calf/giface.h>
 #include <calf/modules_dist.h>
 #include <string>
+
+FORWARD_DECLARE_METADATA(saturator)
+FORWARD_DECLARE_METADATA(exciter)
+FORWARD_DECLARE_METADATA(bassenhancer)
+FORWARD_DECLARE_METADATA(vinyl)
+FORWARD_DECLARE_METADATA(tapesimulator)
+FORWARD_DECLARE_METADATA(crusher)
+FORWARD_DECLARE_METADATA(psyclipper)
+
 #ifdef _MSC_VER
 #include <intrin.h>
 inline int clz(unsigned int value)
@@ -149,12 +158,12 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
     if(bypassed) {
         // everything bypassed
         while(offset < numsamples) {
-            if(in_count > 1 && out_count > 1) {
+            if(ins[1] && outs[1]) {
                 outs[0][offset] = ins[0][offset];
                 outs[1][offset] = ins[1][offset];
-            } else if(in_count > 1) {
+            } else if(ins[1]) {
                 outs[0][offset] = (ins[0][offset] + ins[1][offset]) / 2;
-            } else if(out_count > 1) {
+            } else if(outs[1]) {
                 outs[0][offset] = ins[0][offset];
                 outs[1][offset] = ins[0][offset];
             } else {
@@ -172,7 +181,7 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
             // cycle through samples
             float out[2], in[2] = {0.f, 0.f};
             int c = 0;
-            if(in_count > 1 && out_count > 1) {
+            if(ins[1] && outs[1]) {
                 // stereo in/stereo out
                 // handle full stereo
                 in[0] = ins[0][offset];
@@ -216,13 +225,13 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
                     proc[i] *= 1 + (*params[param_level_in] - 1) / 32;
             }
             
-            if(in_count > 1 && out_count > 1) {
+            if(ins[1] && outs[1]) {
                 // full stereo
                 out[0] = ((proc[0] * *params[param_mix]) + in[0] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[0][offset] = out[0];
                 out[1] = ((proc[1] * *params[param_mix]) + in[1] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[1][offset] = out[1];
-            } else if(out_count > 1) {
+            } else if(outs[1]) {
                 // mono -> pseudo stereo
                 out[0] = ((proc[0] * *params[param_mix]) + in[0] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[0][offset] = out[0];
@@ -234,7 +243,7 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
                 out[0] = ((proc[0] * *params[param_mix]) + in[0] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[0][offset] = out[0];
             }
-            float values[] = {in[0],  in[1], out[0], out[1]};
+            float values[] = {in[0], in[1], out[0], out[1]};
             meters.process(values);
 
             // next sample
@@ -260,7 +269,7 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
         hp[1][3].sanitize();
         p[0].sanitize();
         p[1].sanitize();
-        bypass.crossfade(ins, outs, 2, orig_offset, orig_numsamples);
+        bypass.crossfade(ins, outs, 1 + (int)(ins[1] && outs[1]), orig_offset, orig_numsamples);
     }
     meters.fall(numsamples);
     return outputs_mask;
@@ -341,12 +350,12 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
     if(bypassed) {
         // everything bypassed
         while(offset < numsamples) {
-            if(in_count > 1 && out_count > 1) {
+            if(ins[1] && outs[1]) {
                 outs[0][offset] = ins[0][offset];
                 outs[1][offset] = ins[1][offset];
-            } else if(in_count > 1) {
+            } else if(ins[1]) {
                 outs[0][offset] = (ins[0][offset] + ins[1][offset]) / 2;
-            } else if(out_count > 1) {
+            } else if(outs[1]) {
                 outs[0][offset] = ins[0][offset];
                 outs[1][offset] = ins[0][offset];
             } else {
@@ -372,7 +381,7 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
             float maxDrive = 0.f;
             int c = 0;
             
-            if(in_count > 1 && out_count > 1) {
+            if(ins[1] && outs[1]) {
                 // stereo in/stereo out
                 // handle full stereo
                 in[0] = ins[0][offset];
@@ -411,14 +420,14 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
             }
             maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             
-            if(in_count > 1 && out_count > 1) {
+            if(ins[1] && outs[1]) {
                 maxDrive = std::max(maxDrive, dist[1].get_distortion_level() * *params[param_amount]);
                 // full stereo
                 out[0] = (proc[0] * *params[param_amount] + in2out * in[0]) * *params[param_level_out];
                 out[1] = (proc[1] * *params[param_amount] + in2out * in[1]) * *params[param_level_out];
                 outs[0][offset] = out[0];
                 outs[1][offset] = out[1];
-            } else if(out_count > 1) {
+            } else if(outs[1]) {
                 // mono -> pseudo stereo
                 out[1] = out[0] = (proc[0] * *params[param_amount] + in2out * in[0]) * *params[param_level_out];
                 outs[0][offset] = out[0];
@@ -434,7 +443,7 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
             // next sample
             ++offset;
         } // cycle trough samples
-        bypass.crossfade(ins, outs, 2, orig_offset, orig_numsamples);
+        bypass.crossfade(ins, outs, 1 + (int)(ins[1] && outs[1]), orig_offset, orig_numsamples);
         // clean up
         hp[0][0].sanitize();
         hp[1][0].sanitize();
@@ -602,7 +611,8 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
                     out[1] = proc[1] * *params[param_amount] * *params[param_level_out];
                 else
                     out[1] = (proc[1] * *params[param_amount] + in[1]) * *params[param_level_out];
-                outs[1][offset] = out[1];
+                if(outs[1])
+                    outs[1][offset] = out[1];
                 maxDrive = std::max(dist[0].get_distortion_level() * *params[param_amount],
                                             dist[1].get_distortion_level() * *params[param_amount]);
             } else if(out_count > 1) {
@@ -613,7 +623,8 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
                     out[0] = (proc[0] * *params[param_amount] + in[0]) * *params[param_level_out];
                 outs[0][offset] = out[0];
                 out[1] = out[0];
-                outs[1][offset] = out[1];
+                if(outs[1])
+                    outs[1][offset] = out[1];
                 maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             } else {
                 // stereo -> mono
@@ -631,7 +642,7 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
             // next sample
             ++offset;
         } // cycle trough samples
-        bypass.crossfade(ins, outs, 2, orig_offset, orig_numsamples);
+        bypass.crossfade(ins, outs, 1 + (int)(ins[1] && outs[1]), orig_offset, orig_numsamples);
         // clean up
         lp[0][0].sanitize();
         lp[1][0].sanitize();
@@ -733,10 +744,11 @@ uint32_t vinyl_audio_module::process(uint32_t offset, uint32_t numsamples, uint3
     }
     for(uint32_t i = offset; i < offset + numsamples; i++) {
         float L = ins[0][i];
-        float R = ins[1][i];
+        float R = ins[ins[1]?1:0][i];
         if(bypassed) {
             outs[0][i]  = ins[0][i];
-            outs[1][i]  = ins[1][i];
+            if(outs[1])
+                outs[1][i]  = ins[ins[1]?1:0][i];
             float values[] = {0, 0, 0, 0};
             meters.process(values);
         } else {
@@ -782,7 +794,8 @@ uint32_t vinyl_audio_module::process(uint32_t offset, uint32_t numsamples, uint3
             
             // output
             outs[0][i] = L;
-            outs[1][i] = R;
+            if(outs[1])
+                outs[1][i] = R;
             
             // sanitize filters
             if (*params[param_aging] > 0.f) {
@@ -792,12 +805,12 @@ uint32_t vinyl_audio_module::process(uint32_t offset, uint32_t numsamples, uint3
                 }
             }
             
-            float values[] = {inL, inR, outs[0][i], outs[1][i]};
+            float values[] = {inL, inR, outs[0][i], outs[1] ? outs[1][i]: outs[0][i]};
             meters.process(values);
         }
     }
     if (bypassed)
-        bypass.crossfade(ins, outs, 2, orig_offset, numsamples);
+        bypass.crossfade(ins, outs, 1 + (int)(ins[1] && outs[1]), orig_offset, numsamples);
     meters.fall(numsamples);
     return outputs_mask;
 }
@@ -823,7 +836,7 @@ void vinyl_audio_module::post_instantiate(uint32_t sr)
     settings = new_fluid_settings();
     fluid_settings_setnum(settings, "synth.sample-rate", sr);
     fluid_settings_setint(settings, "synth.polyphony", 32);
-    fluid_settings_setint(settings, "synth.midi-channels", _synths);
+    fluid_settings_setint(settings, "synth.midi-channels", _synths > 16 ? _synths : 16);  // = max(16, _synths)
     fluid_settings_setint(settings, "synth.reverb.active", 0);
     fluid_settings_setint(settings, "synth.chorus.active", 0);
     
@@ -935,12 +948,13 @@ uint32_t tapesimulator_audio_module::process(uint32_t offset, uint32_t numsample
     uint32_t orig_offset = offset;
     for(uint32_t i = offset; i < offset + numsamples; i++) {
         float L = ins[0][i];
-        float R = ins[1][i];
+        float R = ins[ins[1]?1:0][i];
         float Lin = ins[0][i];
-        float Rin = ins[1][i];
+        float Rin = ins[ins[1]?1:0][i];
         if(bypassed) {
             outs[0][i]  = ins[0][i];
-            outs[1][i]  = ins[1][i];
+            if(outs[1])
+                outs[1][i]  = ins[ins[1]?1:0][i];
             float values[] = {0, 0, 0, 0};
             meters.process(values);
         } else {
@@ -1033,7 +1047,8 @@ uint32_t tapesimulator_audio_module::process(uint32_t offset, uint32_t numsample
             
             // output
             outs[0][i] = L;
-            outs[1][i] = R;
+            if(outs[1])
+                outs[1][i] = R;
             
             // sanitize filters
             lp[0][0].sanitize();
@@ -1049,12 +1064,12 @@ uint32_t tapesimulator_audio_module::process(uint32_t offset, uint32_t numsample
             rms = std::max((double)rms, (double)((fabs(Lo) + fabs(Ro)) / 2));
             input = std::max((double)input, (double)((fabs(Lc) + fabs(Rc)) / 2));
             
-            float values[] = {inL, inR, outs[0][i], outs[1][i]};
+            float values[] = {inL, inR, outs[0][i], outs[outs[1]?1:0][i]};
             meters.process(values);
         }
     }
     if (bypassed)
-        bypass.crossfade(ins, outs, 2, orig_offset, numsamples);
+        bypass.crossfade(ins, outs, 1 + (int)(ins[1] && outs[1]), orig_offset, numsamples);
     meters.fall(numsamples);
     return outputs_mask;
 }
@@ -1220,7 +1235,8 @@ uint32_t crusher_audio_module::process(uint32_t offset, uint32_t numsamples, uin
         // everything bypassed
         while(offset < numsamples) {
             outs[0][offset] = ins[0][offset];
-            outs[1][offset] = ins[1][offset];
+            if(outs[1])
+                outs[1][offset] = ins[ins[1]?1:0][offset];
             float values[] = {0, 0, 0, 0};
             meters.process(values);
             ++offset;
@@ -1236,19 +1252,27 @@ uint32_t crusher_audio_module::process(uint32_t offset, uint32_t numsamples, uin
                 samplereduction[1].set_params(smin + sdiff * (lfo.get_value() + 0.5));
             }
             outs[0][offset] = samplereduction[0].process(ins[0][offset] * *params[param_level_in]);
-            outs[1][offset] = samplereduction[1].process(ins[1][offset] * *params[param_level_in]);
             outs[0][offset] = outs[0][offset] * *params[param_morph] + ins[0][offset] * (*params[param_morph] * -1 + 1) * *params[param_level_in];
-            outs[1][offset] = outs[1][offset] * *params[param_morph] + ins[1][offset] * (*params[param_morph] * -1 + 1) * *params[param_level_in];
             outs[0][offset] = bitreduction.process(outs[0][offset]) * *params[param_level_out];
-            outs[1][offset] = bitreduction.process(outs[1][offset]) * *params[param_level_out];
-            float values[] = {ins[0][offset], ins[1][offset], outs[0][offset], outs[1][offset]};
+            if(outs[1] && ins[1]) {
+                const float* ins1 = ins[1] ? ins[1] : ins[0];
+                outs[1][offset] = samplereduction[1].process(ins1[offset] * *params[param_level_in]);
+                outs[1][offset] = outs[1][offset] * *params[param_morph] + ins1[offset] * (*params[param_morph] * -1 + 1) * *params[param_level_in];
+                outs[1][offset] = bitreduction.process(outs[1][offset]) * *params[param_level_out];
+            }
+            float values[] = {
+                ins[0][offset],
+                ins[ins[1]?1:0][offset],
+                outs[0][offset],
+                outs[outs[1]?1:0][offset]
+            };
             meters.process(values);
             // next sample
             ++offset;
             if (*params[param_lforate])
                 lfo.advance(1);
         } // cycle trough samples
-        bypass.crossfade(ins, outs, 2, orig_offset, orig_numsamples);
+        bypass.crossfade(ins, outs, 1 + (int)(ins[1] && outs[1]), orig_offset, orig_numsamples);
     }
     meters.fall(numsamples);
     return outputs_mask;
@@ -1265,3 +1289,157 @@ bool crusher_audio_module::get_gridline(int index, int subindex, int phase, floa
 {
     return bitreduction.get_gridline(subindex, phase, pos, vertical, legend, context);
 }
+
+/**********************************************************************
+ * PSYCHOACOUSTIC CLIPPER by Jason Jang
+**********************************************************************/
+
+psyclipper_audio_module::psyclipper_audio_module()
+{
+    is_active = false;
+    srate = 0;
+    // need to know sample rate before we can create the clippers
+    clipper[0] = NULL;
+    clipper[1] = NULL;
+}
+
+psyclipper_audio_module::~psyclipper_audio_module()
+{
+    for(int i = 0; i < 2; i++) {
+        if(clipper[i]) {
+            delete clipper[i];
+        }
+    }
+}
+
+void psyclipper_audio_module::activate()
+{
+    is_active = true;
+    // set all filters and strips
+    params_changed();
+}
+
+void psyclipper_audio_module::deactivate()
+{
+    is_active = false;
+}
+
+void psyclipper_audio_module::params_changed()
+{
+    bool bypass = *params[param_bypass] > 0.5f;
+    int iterations = bypass ? 0 : *params[param_iterations];
+    for(int i = 0; i < 2; i++) {
+        clipper[i]->set_clip_level(*params[param_limit]);
+        clipper[i]->set_iterations(iterations);
+        clipper[i]->set_adaptive_distortion_strength(*params[param_adaptive_distortion]);
+    }
+
+    int margin_curve[10][2] = {
+        {0, (int)*params[param_protection125],},
+        {125, (int)*params[param_protection125]},
+        {250, (int)*params[param_protection250]},
+        {500, (int)*params[param_protection500]},
+        {1000, (int)*params[param_protection1000]},
+        {2000, (int)*params[param_protection2000]},
+        {4000, (int)*params[param_protection4000]},
+        {8000, (int)*params[param_protection8000]},
+        {16000, (int)*params[param_protection16000]},
+        {20000, -10},
+    };
+    if(memcmp(old_margin_curve, margin_curve, sizeof(old_margin_curve))) {
+        memcpy(old_margin_curve, margin_curve, sizeof(old_margin_curve));
+        for(int i = 0; i < 2; i++) {
+            clipper[i]->set_margin_curve(margin_curve, 10);
+        }
+    }
+}
+
+void psyclipper_audio_module::set_sample_rate(uint32_t sr)
+{
+    int meter[] = {param_meter_inL, param_meter_inR,  param_meter_outL, param_meter_outR, param_margin_shift};
+    int clip[] = {param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR, -1};
+    meters.init(params, meter, clip, 5, sr);
+
+    if(!clipper[0] || sr != srate) {
+        int fftSize = sr > 100000 ? 1024 : sr > 50000 ? 512 : 256;
+        for(int i = 0; i < 2; i++) {
+            if(clipper[i]) {
+                delete clipper[i];
+            }
+            clipper[i] = new shaping_clipper(sr, fftSize, 1.0);
+            in_buffer[i].resize(clipper[i]->get_feed_size());
+            out_buffer[i].resize(clipper[i]->get_feed_size());
+        }
+        buffer_offset = 0;
+    }
+    srate = sr;
+}
+
+uint32_t psyclipper_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
+{
+    bool bypassed = bypass.update(*params[param_bypass] > 0.5f, numsamples);
+    bool diff_only = *params[param_diff_only] > 0.5f;
+    numsamples += offset;
+    if(bypassed || !clipper[0]) {
+        // everything bypassed
+        while(offset < numsamples) {
+            outs[0][offset] = ins[0][offset];
+            outs[1][offset] = ins[1][offset];
+            float values[] = {0, 0, 0, 0, 1};
+            meters.process(values);
+            ++offset;
+        }
+    } else {
+
+        while(offset < numsamples) {
+            int samples_to_process = std::min<int>(clipper[0]->get_feed_size() - buffer_offset, numsamples - offset);
+            for(int i = 0; i < samples_to_process; i++) {
+                float inL = ins[0][offset];
+                float inR = ins[1][offset];
+
+                // in level
+                inR *= *params[param_level_in];
+                inL *= *params[param_level_in];
+
+                // add to input buffer
+                in_buffer[0][buffer_offset] = inL;
+                in_buffer[1][buffer_offset] = inR;
+
+                // take from output buffer
+                float outL = out_buffer[0][buffer_offset];
+                float outR = out_buffer[1][buffer_offset];
+
+                // autolevel
+                if (*params[param_auto_level]) {
+                    outL /= *params[param_limit];
+                    outR /= *params[param_limit];
+                }
+
+                // out level
+                outL *= *params[param_level_out];
+                outR *= *params[param_level_out];
+
+                // send to output
+                outs[0][offset] = outL;
+                outs[1][offset] = outR;
+                buffer_offset++;
+                offset++;
+
+                float values[] = {inL, inR, outL, outR, last_margin_shift};
+                meters.process (values);
+
+            }
+            if(buffer_offset == clipper[0]->get_feed_size()) {
+                // just filled the input buffer and emptied the output buffer
+                float margin_shift_left, margin_shift_right;
+                clipper[0]->feed(in_buffer[0].data(), out_buffer[0].data(), diff_only, &margin_shift_left);
+                clipper[1]->feed(in_buffer[1].data(), out_buffer[1].data(), diff_only, &margin_shift_right);
+                buffer_offset = 0;
+                last_margin_shift = 1.0 / std::max(margin_shift_left, margin_shift_right);
+            }
+        }
+    } // process (no bypass)
+    meters.fall(numsamples);
+    return outputs_mask;
+}
+
