@@ -27,6 +27,19 @@
 using namespace dsp;
 using namespace calf_plugins;
 
+FORWARD_DECLARE_METADATA(equalizer5band)
+FORWARD_DECLARE_METADATA(equalizer8band)
+FORWARD_DECLARE_METADATA(equalizer12band)
+FORWARD_DECLARE_METADATA(equalizer30band)
+FORWARD_DECLARE_METADATA(envelopefilter)
+FORWARD_DECLARE_METADATA(filter)
+FORWARD_DECLARE_METADATA(filterclavier)
+FORWARD_DECLARE_METADATA(emphasis)
+FORWARD_DECLARE_METADATA(xover2)
+FORWARD_DECLARE_METADATA(xover3)
+FORWARD_DECLARE_METADATA(xover4)
+FORWARD_DECLARE_METADATA(vocoder)
+
 #define SET_IF_CONNECTED(name) if (params[AM::param_##name] != NULL) *params[AM::param_##name] = name;
 
 /**********************************************************************
@@ -633,7 +646,7 @@ void equalizer30band_audio_module::params_changed()
         eq_arrR[*params[param_filters]]->changeBandGainDb(i,*params[psr + band_params*i]);
     }
 
-    //Upadte filter type
+    //Update filter type
     flt_type = (filter_type)int((*params[param_filters] + 1));
 }
 
@@ -645,7 +658,7 @@ void equalizer30band_audio_module::set_sample_rate(uint32_t sr)
     for(unsigned int i = 0; i < eq_arrL.size(); i++)
     {
         eq_arrL[i]->setSampleRate(srate);
-        eq_arrL[i]->setSampleRate(srate);
+        eq_arrR[i]->setSampleRate(srate);//maybe a typo flaw, by vlee78
     }
 
     int meter[] = {param_level_in_vuL, param_level_in_vuR, param_level_out_vuL, param_level_out_vuR};
@@ -721,8 +734,7 @@ uint32_t equalizer30band_audio_module::process(uint32_t offset, uint32_t numsamp
 **********************************************************************/
 
 filterclavier_audio_module::filterclavier_audio_module() 
-: filter_module_with_inertia<biquad_filter_module, filterclavier_metadata>(ins, outs, params)
-, min_gain(1.0)
+: min_gain(1.0)
 , max_gain(32.0)
 , last_note(-1)
 , last_velocity(-1)
@@ -775,7 +787,7 @@ void filterclavier_audio_module::note_on(int channel, int note, int vel)
             (float(vel) / 127.0) 
             // 0.001: if the difference is equal to zero (which happens
             // when the max_resonance knom is at minimum position
-            // then the filter gain doesnt seem to snap to zero on most note offs
+            // then the filter gain doesn't seem to snap to zero on most note offs
             * (*params[par_max_resonance] - min_resonance + 0.001) 
             + min_resonance);
     
@@ -1039,7 +1051,7 @@ uint32_t xover_audio_module<XoverBaseClass>::process(uint32_t offset, uint32_t n
                 // fill delay buffer
                 buffer[pos + ptr] = xval;
                 
-                // get value from delay buffer if neccessary
+                // get value from delay buffer if necessary
                 if (*params[AM::param_delay1 + off])
                     xval = buffer[(pos - (int)nbuf + ptr + buffer_size) % buffer_size];
                 
