@@ -15,24 +15,30 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  */
-#include <config.h>
-#include <assert.h>
-#include <memory.h>
-#if USE_JACK
-#include <jack/jack.h>
-#endif
-#include <calf/giface.h>
 #include <calf/synth.h>
 
 using namespace dsp;
 using namespace std;
 
+void basic_synth::init_voices(int count)
+{
+    allocated_voices.init(count);
+    active_voices.init(count);
+    unused_voices.init(count);
+    for (int i = 0; i < count; i++)
+    {
+        dsp::voice *v = alloc_voice();
+        allocated_voices.add(v);
+        unused_voices.add(v);
+    }
+}
+
 void basic_synth::kill_note(int note, int vel, bool just_one)
 {
-    for (list<dsp::voice *>::iterator it = active_voices.begin(); it != active_voices.end(); it++) {
+    for_all_voices(it) {
         // preserve sostenuto notes
         if ((*it)->get_current_note() == note && !(sostenuto && (*it)->sostenuto)) {
             (*it)->note_off(vel);
@@ -45,48 +51,42 @@ void basic_synth::kill_note(int note, int vel, bool just_one)
 dsp::voice *basic_synth::give_voice()
 {
     if (active_voices.size() >= polyphony_limit)
-    {
-        dsp::voice *stolen = steal_voice();
-        if (stolen)
-            return stolen;
-    }
+        steal_voice();
     if (unused_voices.empty())
-        return alloc_voice();
+        return NULL;
     else {
-        dsp::voice *v = unused_voices.top();
-        unused_voices.pop();
+        dsp::voice *v = unused_voices.pop();
         v->reset();
         return v;
     }   
 }
 
-dsp::voice *basic_synth::steal_voice()
+void basic_synth::steal_voice()
 {
-    std::list<dsp::voice *>::iterator found = active_voices.end();
+    dsp::voice *found = NULL;
     float priority = 10000;
     //int idx = 0;
-    for(std::list<dsp::voice *>::iterator i = active_voices.begin(); i != active_voices.end(); i++)
+    for_all_voices(i)
     {
         //printf("Voice %d priority %f at %p\n", idx++, (*i)->get_priority(), *i);
         if ((*i)->get_priority() < priority)
         {
             priority = (*i)->get_priority();
-            found = i;
+            found = *i;
         }
     }
     //printf("Found: %p\n\n", *found);
-    if (found == active_voices.end())
-        return NULL;
+    if (!found)
+        return;
     
-    (*found)->steal();
-    return NULL;
+    found->steal();
 }
 
 void basic_synth::trim_voices()
 {
     // count stealable voices
     unsigned int count = 0;
-    for(std::list<dsp::voice *>::iterator i = active_voices.begin(); i != active_voices.end(); i++)
+    for_all_voices(i)
     {
         if ((*i)->get_priority() < 10000)
             count++;
@@ -107,12 +107,14 @@ void basic_synth::note_on(int note, int vel)
     }
     bool perc = check_percussion();
     dsp::voice *v = give_voice();
+    if (!v)
+        return;
     v->setup(sample_rate);
     v->released = false;
     v->sostenuto = false;
     gate.set(note);
     v->note_on(note, vel);
-    active_voices.push_back(v);
+    active_voices.add(v);
     if (perc) {
         percussion_note_on(note, vel);
     }
@@ -125,8 +127,6 @@ void basic_synth::note_off(int note, int vel)
         kill_note(note, vel, false);
 }
 
-#define for_all_voices(iter) for (std::list<dsp::voice *>::iterator iter = active_voices.begin(); iter != active_voices.end(); iter++)
-    
 void basic_synth::on_pedal_release()
 {
     for_all_voices(i)
@@ -177,8 +177,6 @@ void basic_synth::control_change(int ctl, int val)
         }
     }
     if (ctl == 123 || ctl == 120) { // all notes off, all sounds off
-        vector<int> notes;
-        notes.reserve(128);
         if (ctl == 120) { // for "all sounds off", automatically release hold and sostenuto pedal
             control_change(66, 0);
             control_change(64, 0);
@@ -205,12 +203,12 @@ void basic_synth::control_change(int ctl, int val)
 void basic_synth::render_to(float (*output)[2], int nsamples)
 {
     // render voices, eliminate ones that aren't sounding anymore
-    for (list<dsp::voice *>::iterator i = active_voices.begin(); i != active_voices.end();) {
+    for (dsp::voice **i = active_voices.begin(); i != active_voices.end(); ) {
         dsp::voice *v = *i;
         v->render_to(output, nsamples);
         if (!v->get_active()) {
             i = active_voices.erase(i);
-            unused_voices.push(v);
+            unused_voices.add(v);
             continue;
         }
         i++;
@@ -219,11 +217,7 @@ void basic_synth::render_to(float (*output)[2], int nsamples)
 
 basic_synth::~basic_synth()
 {
-    while(!unused_voices.empty()) {
-        delete unused_voices.top();
-        unused_voices.pop();
-    }
-    for (list<voice *>::iterator i = active_voices.begin(); i != active_voices.end(); i++)
+    for (voice_array::iterator i = allocated_voices.begin(); i != allocated_voices.end(); ++i)
         delete *i;
 }
 
